WsSettings = {}

WsSettings.TypeMap = {
    boolean = "bool",
	number  = "float",
	string  = "string"
}

local WsSettings_mt = Class(WsSettings)

function WsSettings:new(modName, fileName, useSaveGameDirectory, loadHookFunction, saveHookFunction, defaults,logger)
    local self=setmetatable( {}, WsSettings_mt)
    self.modName=modName
    self.fileName=fileName
    self.useSaveGameDirectory=useSaveGameDirectory
    self.loadHookFunction=loadHookFunction
    self.saveHookFunction=saveHookFunction
    self.logger=logger
    self.settings={}
    self.defaults={}

    self:addDefaults(defaults)
    if self.fileName:sub(-4) ~= ".xml" then
		self.fileName = self.fileName .. ".xml"
	end
    return self
end
function WsSettings:addDefaults(defaults)
    if type(defaults)=="table" then
        for key,value in pairs(defaults) do
            if type(value)=="table" then
                self.defaults[key]=value
            else
                self.defaults[key]= { value, WsSettings.TypeMap[type(value)]}
            end
        end
    else
        self.logger:print("WsSettings:addDefaults only supports tables", WsLogger.Severity.Error)
    end
end
function WsSettings:getValue(name)
    if self.settings[name] == nil then
        if self.defaults[name]==nil or self.defaults[name][1]==nil then
            self.logger:printVariable(name,"WsSettings:getValue unknown setting:", WsLogger.Severity.Warning)
            return nil
        else
            self.logger:printVariable(name,"WsSettings:getValue not set setting, returning default:",WsLogger.Severity.Verbose)
            return self.defaults[name][1]
        end
    else
        self.logger:printVariable(name,"WsSettings:getValue found setting:",WsLogger.Severity.Verbose)
        return self.settings[name]
    end
end
function WsSettings:setValue(name,value)
    if self.settings[name]==nil and self.defaults[name]==nil then
        self.logger:printVariable(name,"WsSettings:setValue unknown setting:", WsLogger.Severity.Warning)
        return nil
    end
    self.settings[name]=value
    self.logger:printVariable(name,"WsSettings:setValue set variable:", WsLogger.Severity.Verbose)
    return self:getValue(name)
end
function WsSettings:loadSettings()
    self.logger:print("WsSettings:loadSettings called", WsLogger.Severity.Call)
    if fileExists(self:xmlFilePath()) then
        self.logger:print("WsSettings:loadSettings file exists", WsLogger.Severity.Debug)
        local xmlFile=loadXMLFile(self.modName,self:xmlFilePath())
        for key,value in pairs(self.defaults) do
            self.logger:printVariable(key,"WsSettings:loadSettings actual key:",WsLogger.Severity.Debug)
            self.logger:printVariable(value,"WsSettings:loadSettings actual value:",WsLogger.Severity.Debug)
            if value[2]=="bool" then
                self:setValue(
					key,
					Utils.getNoNil(getXMLBool(
						xmlFile,
						self:xmlPath(self.modName,key,"value")
					), value[1])
				)
            elseif value[2]=="string" then
                self:setValue(
					key,
					Utils.getNoNil(getXMLString(
						xmlFile,
						self:xmlPath(self.modName,key,"value")
					), value[1])
				)
            elseif value[2]=="int" then
                self:setValue(
					key,
					Utils.getNoNil(getXMLInt(
						xmlFile,
						self:xmlPath(self.modName,key,"value")
					), value[1])
				)
            elseif value[2]=="float" then
                self:setValue(
					key,
					Utils.getNoNil(getXMLFloat(
						xmlFile,
						self:xmlPath(self.modName,key,"value")
					), value[1])
				)
            end
        end
        delete(xmlFile)
    else
        self.logger:print("WsSettings:loadSettings file does not exist, returning", WsLogger.Severity.Verbose)
    end
    self.logger:print("WsSettings:loadSettings xml loaded",WsLogger.Severity.Debug)
    if type(self.loadHookFunction) == "function" then
        self.loadHookFunction()
    end
end
function WsSettings:xmlPath(key, element, attrib)
    return key.."."..element.."#"..attrib
end
function WsSettings:saveSettings()
    self.logger:print("WsSettings:saveSettings called", WsLogger.Severity.Call)
    local folder=self:xmlFolderPath()
    if ( not fileExists(folder) ) then createFolder(folder) end
    local file=self:xmlFilePath()
    local xmlFile=createXMLFile(self.modName,file,self.modName)
    for key,value in pairs(self.defaults) do
        self.logger:printVariable(key,"WsSettings:saveSettings actual key:",WsLogger.Severity.Debug)
        self.logger:printVariable(self:getValue(key),"WsSettings:saveSettings actual value:",WsLogger.Severity.Debug)
        if value[2]=="bool" then
            setXMLBool(xmlFile,self:xmlPath(self.modName,key,"value"),self:getValue(key))
        elseif value[2]=="string" then
            setXMLString(xmlFile,self:xmlPath(self.modName,key,"value"),self:getValue(key))
        elseif value[2]=="int" then
            setXMLInt(xmlFile,self:xmlPath(self.modName,key,"value"),self:getValue(key))
        elseif value[2]=="float" then
            setXMLFloat(xmlFile,self:xmlPath(self.modName,key,"value"),self:getValue(key))
        end
    end
    saveXMLFile(xmlFile)
    delete(xmlFile)
    self.logger:print("WsSettings:saveSettings xml saved",WsLogger.Severity.Debug)
    if type(self.saveHookFunction) == "function" then
        self.saveHookFunction()
    end


end
function WsSettings:xmlFolderPath()
    self.logger:print("WsSettings:xmlFolderPath called", WsLogger.Severity.Call)
    local folder=""
    if self.useSaveGameDirectory then
        self.logger:print("WsSettings:xmlFolderPath useSaveGameDirectory", WsLogger.Severity.Call)
        folder=('%ssavegame%d/'):format(getUserProfileAppPath(),g_currentMission.missionInfo.savegameIndex)
    else
        self.logger:print("WsSettings:xmlFolderPath not useSaveGameDirectory", WsLogger.Severity.Call)
        folder=('%smodSettings/%s/'):format(getUserProfileAppPath(),self.modName)
    end
    self.logger:printVariable(folder,"WsSettings:xmlFolderPath Folder:",WsLogger.Severity.Debug)
    return folder
end
function WsSettings:xmlFilePath()
    self.logger:print("WsSettings:xmlFilePath called", WsLogger.Severity.Call)
    local filepath=self:xmlFolderPath()
    if self.useSaveGameDirectory then
        filepath=filepath..self.modName..".xml"
    else
        filepath=filepath..self.filename
    end
    self.logger:printVariable(filepath,"WsSettings:xmlFilePath Path:",WsLogger.Severity.Debug)
    return filepath
end